@@ -2,29 +2,31 @@
 
 namespace MabeEnumTest;
 
+use InvalidArgumentException;
 use MabeEnum\EnumSet;
 use MabeEnumTest\TestAsset\EmptyEnum;
+use MabeEnumTest\TestAsset\Enum31;
 use MabeEnumTest\TestAsset\EnumBasic;
 use MabeEnumTest\TestAsset\EnumInheritance;
 use MabeEnumTest\TestAsset\Enum32;
 use MabeEnumTest\TestAsset\Enum64;
 use MabeEnumTest\TestAsset\Enum65;
 use MabeEnumTest\TestAsset\Enum66;
-use PHPUnit_Framework_TestCase as TestCase;
+use PHPUnit\Framework\TestCase;
 
 /**
  * Unit tests for the class MabeEnum\EnumSet
  *
  * @link http://github.com/marc-mabe/php-enum for the canonical source repository
- * @copyright Copyright (c) 2015 Marc Bennewitz
+ * @copyright Copyright (c) 2017 Marc Bennewitz
  * @license http://github.com/marc-mabe/php-enum/blob/master/LICENSE.txt New BSD License
  */
 class EnumSetTest extends TestCase
 {
     public function testBasic()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $this->assertSame('MabeEnumTest\TestAsset\EnumBasic', $enumSet->getEnumeration());
+        $enumSet = new EnumSet(EnumBasic::class);
+        $this->assertSame(EnumBasic::class, $enumSet->getEnumeration());
 
         $enum1  = EnumBasic::ONE();
         $enum2  = EnumBasic::TWO();
@@ -44,15 +46,9 @@ class EnumSetTest extends TestCase
         $this->assertFalse($enumSet->contains($enum2));
     }
 
-    public function testDeprecatedGetEnumClass()
-    {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $this->assertSame('MabeEnumTest\TestAsset\EnumBasic', $enumSet->getEnumClass());
-    }
-
     public function testBasicWithConstantValuesAsEnums()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $enumSet = new EnumSet(EnumBasic::class);
 
         $enum1  = EnumBasic::ONE;
         $enum2  = EnumBasic::TWO;
@@ -74,7 +70,7 @@ class EnumSetTest extends TestCase
 
     public function testUnique()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $enumSet = new EnumSet(EnumBasic::class);
 
         $enumSet->attach(EnumBasic::ONE());
         $enumSet->attach(EnumBasic::ONE);
@@ -87,7 +83,7 @@ class EnumSetTest extends TestCase
 
     public function testIterateOrdered()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $enumSet = new EnumSet(EnumBasic::class);
 
         // an empty enum set needs to be invalid, starting by 0
         $this->assertSame(0, $enumSet->count());
@@ -127,7 +123,7 @@ class EnumSetTest extends TestCase
 
     public function testIterateAndDetach()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\EnumInheritance');
+        $enumSet = new EnumSet(EnumInheritance::class);
 
         $enum1 = EnumInheritance::ONE();
         $enum2 = EnumInheritance::TWO();
@@ -161,20 +157,20 @@ class EnumSetTest extends TestCase
 
     public function testConstructThrowsInvalidArgumentExceptionIfEnumClassDoesNotExtendBaseEnum()
     {
-        $this->setExpectedException('InvalidArgumentException');
-        new EnumSet('stdClass');
+        $this->expectException(InvalidArgumentException::class);
+        new EnumSet(self::class);
     }
 
     public function testInitEnumThrowsInvalidArgumentExceptionOnInvalidEnum()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $this->setExpectedException('InvalidArgumentException');
+        $enumSet = new EnumSet(EnumBasic::class);
+        $this->expectException(InvalidArgumentException::class);
         $this->assertFalse($enumSet->contains(EnumInheritance::INHERITANCE()));
     }
 
     public function testIterateOutOfRangeIfLastOrdinalEnumIsSet()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $enumSet = new EnumSet(EnumBasic::class);
         $enum    = EnumBasic::byOrdinal(count(EnumBasic::getConstants()) - 1);
 
         $enumSet->attach($enum);
@@ -192,9 +188,9 @@ class EnumSetTest extends TestCase
         $this->assertSame($enum->getOrdinal() + 1, $enumSet->key());
     }
 
-    public function testRewindFirstOnEmptySet()
+    public function testRewindIntFirstOnEmptySet()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $enumSet = new EnumSet(EnumBasic::class);
 
         $enumSet->attach(EnumBasic::TWO);
         $enumSet->rewind();
@@ -205,9 +201,22 @@ class EnumSetTest extends TestCase
         $this->assertSame(0, $enumSet->key());
     }
 
+    public function testRewindBinFirstOnEmptySet()
+    {
+        $enumSet = new EnumSet(Enum66::class);
+
+        $enumSet->attach(Enum66::TWO);
+        $enumSet->rewind();
+        $this->assertGreaterThan(0, $enumSet->key());
+
+        $enumSet->detach(Enum66::TWO);
+        $enumSet->rewind();
+        $this->assertSame(0, $enumSet->key());
+    }
+
     public function test32EnumerationsSet()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\Enum32');
+        $enumSet = new EnumSet(Enum32::class);
         foreach (Enum32::getConstants() as $name => $value) {
             $this->assertFalse($enumSet->contains($value));
             $enumSet->attach($value);
@@ -226,7 +235,7 @@ class EnumSetTest extends TestCase
 
     public function test64EnumerationsSet()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\Enum64');
+        $enumSet = new EnumSet(Enum64::class);
         foreach (Enum64::getConstants() as $name => $value) {
             $this->assertFalse($enumSet->contains($value));
             $enumSet->attach($value);
@@ -245,16 +254,52 @@ class EnumSetTest extends TestCase
 
     public function test65EnumerationsSet()
     {
-        $enum = new EnumSet('MabeEnumTest\TestAsset\Enum65');
+        $enum = new EnumSet(Enum65::class);
 
         $this->assertNull($enum->attach(Enum65::byOrdinal(64)));
         $enum->next();
         $this->assertTrue($enum->valid());
     }
 
+    public function testGetBit()
+    {
+        $enumSet = new EnumSet(EnumBasic::class);
+        $enumSet->attach(EnumBasic::TWO);
+
+        $this->assertFalse($enumSet->getBit(EnumBasic::ONE()->getOrdinal()));
+        $this->assertTrue($enumSet->getBit(EnumBasic::TWO()->getOrdinal()));
+    }
+
+    public function testGetBitOutOfRangeOrdinal()
+    {
+        $enumSet = new EnumSet(EnumBasic::class);
+
+        $this->expectException(InvalidArgumentException::class);
+        $enumSet->getBit(100);
+    }
+
+    public function testSetBit()
+    {
+        $enumSet = new EnumSet(EnumBasic::class);
+
+        $enumSet->setBit(EnumBasic::TWO()->getOrdinal(), true);
+        $this->assertTrue($enumSet->getBit(EnumBasic::TWO()->getOrdinal()));
+
+        $enumSet->setBit(EnumBasic::TWO()->getOrdinal(), false);
+        $this->assertFalse($enumSet->getBit(EnumBasic::TWO()->getOrdinal()));
+    }
+
+    public function testSetBitOutOfRangeOrdinal()
+    {
+        $enumSet = new EnumSet(EnumBasic::class);
+
+        $this->expectException(InvalidArgumentException::class);
+        $enumSet->setBit(100, true);
+    }
+
     public function testGetBinaryBitsetLe()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\Enum65');
+        $enumSet = new EnumSet(Enum65::class);
         
         $enum1 = Enum65::ONE;
         $enum2 = Enum65::TWO;
@@ -288,7 +333,7 @@ class EnumSetTest extends TestCase
 
     public function testGetBinaryBitsetBe()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\Enum65');
+        $enumSet = new EnumSet(Enum65::class);
         
         $enum1 = Enum65::ONE;
         $enum2 = Enum65::TWO;
@@ -320,75 +365,118 @@ class EnumSetTest extends TestCase
         $this->assertSame(3, $enumSet->count());
     }
 
-    /**
-     * @deprecated
-     */
-    public function testGetBitset()
+    public function testSetBinaryBitsetLeBin()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\Enum65');
-        
-        $enum1 = Enum65::ONE;
-        $enum2 = Enum65::TWO;
-        $enum3 = Enum65::SIXTYFIVE;
-        $enum4 = Enum65::SIXTYFOUR;
+        $enumSet = new EnumSet(Enum65::class);
+        $enumSet->setBinaryBitsetLe("\x01\x00\x00\x00\x00\x00\x00\x80\x01");
 
-        $this->assertNull($enumSet->attach($enum1));
-        $this->assertSame("\x00\x00\x00\x00\x00\x00\x00\x00\x01", $enumSet->getBitset());
-        $this->assertTrue($enumSet->contains($enum1));
-
-        $this->assertNull($enumSet->attach($enum2));
-        $this->assertSame("\x00\x00\x00\x00\x00\x00\x00\x00\x03", $enumSet->getBitset());
-        $this->assertTrue($enumSet->contains($enum2));
-
-        $this->assertNull($enumSet->attach($enum3));
-        $this->assertSame("\x01\x00\x00\x00\x00\x00\x00\x00\x03", $enumSet->getBitset());
-        $this->assertTrue($enumSet->contains($enum3));
-
-        $this->assertNull($enumSet->attach($enum4));
-        $this->assertSame("\x01\x80\x00\x00\x00\x00\x00\x00\x03", $enumSet->getBitset());
-        $this->assertTrue($enumSet->contains($enum4));
-        
-        $this->assertSame(4, $enumSet->count());
-
-        $this->assertNull($enumSet->detach($enum2));
-        $this->assertSame("\x01\x80\x00\x00\x00\x00\x00\x00\x01", $enumSet->getBitset());
-        $this->assertFalse($enumSet->contains($enum2));
-        
+        $this->assertContains(Enum65::ONE(), $enumSet);
+        $this->assertNotContains(Enum65::TWO(), $enumSet);
+        $this->assertContains(Enum65::SIXTYFIVE(), $enumSet);
+        $this->assertContains(Enum65::SIXTYFOUR(), $enumSet);
         $this->assertSame(3, $enumSet->count());
     }
 
-    public function testSetBinaryBitsetLe()
+    public function testSetBinaryBitsetLeBinShort()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\Enum65');
-        $enumSet->setBinaryBitsetLe("\x01\x00\x00\x00\x00\x00\x00\x80\x01");
-
-        $this->assertTrue($enumSet->contains(Enum65::ONE));
-        $this->assertFalse($enumSet->contains(Enum65::TWO));
-        $this->assertTrue($enumSet->contains(Enum65::SIXTYFIVE));
-        $this->assertTrue($enumSet->contains(Enum65::SIXTYFOUR));
-        $this->assertTrue($enumSet->count() == 3);
+        $enumSet = new EnumSet(Enum65::class);
+        $enumSet->setBinaryBitsetLe("\x0A");
+        $this->assertSame("\x0A\x00\x00\x00\x00\x00\x00\x00\x00", $enumSet->getBinaryBitsetLe());
     }
 
-    public function testSetBinaryBitsetLeTruncateHighBits()
+    public function testSetBinaryBitsetLeBinLong()
     {
-        // using Enum66 to make sure the max. ordinal number gets converted into a bitset
-        // Enum65 has max. ordinal number of 1 of the last byte. -> 00000001
-        // Enum66 has max. ordinal number of 2 of the last byte. -> 00000011
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\Enum66');
-        foreach (Enum66::getEnumerators() as $enumerator) {
-            $enumSet->attach($enumerator);
-        }
+        $enumSet = new EnumSet(Enum65::class);
+        $bitset = "\x0A\xFF\x00\x00\x00\x00\x00\x00\x00";
+        $enumSet->setBinaryBitsetLe($bitset . "\x00\x00\x00\x00\x00\x00\x00");
+        $this->assertSame($bitset, $enumSet->getBinaryBitsetLe());
+    }
 
+    public function testSetBinaryBitsetLeBinOutOfRangeBitsOfExtendedBytes1()
+    {
+        $enumSet = new EnumSet(Enum65::class);
+
+        $this->expectException(InvalidArgumentException::class, 'Out-Of-Range');
+        $enumSet->setBinaryBitsetLe("\xff\xff\xff\xff\xff\xff\xff\xff\x00\x02");
+    }
+
+    public function testSetBinaryBitsetLeBinOutOfRangeBitsOfExtendedBytes2()
+    {
+        $enumSet = new EnumSet(Enum65::class);
+
+        $this->expectException(InvalidArgumentException::class, 'Out-Of-Range');
+        $enumSet->setBinaryBitsetLe("\xff\xff\xff\xff\xff\xff\xff\xff\x02");
+    }
+
+    public function testSetBinaryBitsetLeBinOutOfRangeBitsOfLastValidByte()
+    {
+        // using Enum65 to detect Out-Of-Range bits of last valid byte
+        // Enum65 has max. ordinal number of 2 of the last byte. -> 0001
+        $enumSet   = new EnumSet(Enum65::class);
         $bitset    = $enumSet->getBinaryBitsetLe();
-        $newBitset = substr($bitset, 0, -1) . "\xff\xff";
-        $enumSet->setBinaryBitsetLe($newBitset);
+        $newBitset = substr($bitset, 0, -1) . "\x02";
 
-        $this->assertSame(bin2hex($bitset), bin2hex($enumSet->getBinaryBitsetLe()));
+        $this->expectException(InvalidArgumentException::class, 'Out-Of-Range');
+        $enumSet->setBinaryBitsetLe($newBitset);
+    }
+
+    public function testSetBinaryBitsetLeBinArgumentExceptionIfNotString()
+    {
+        $this->expectException(InvalidArgumentException::class);
+        
+        $enum = new EnumSet(Enum65::class);
+        $enum->setBinaryBitsetLe(0);
+    }
+
+    public function testSetBinaryBitsetLeInt()
+    {
+        $enumSet = new EnumSet(Enum32::class);
+        $enumSet->setBinaryBitsetLe("\x01\x00\x80\x01");
+        $this->assertContains(Enum32::ONE(), $enumSet);
+        $this->assertNotContains(Enum32::TWO(), $enumSet);
+        $this->assertContains(Enum32::TWENTYFOUR(), $enumSet);
+        $this->assertContains(Enum32::TWENTYFIVE(), $enumSet);
+        $this->assertSame(3, $enumSet->count());
+    }
+
+    public function testSetBinaryBitsetLeIntShort()
+    {
+        $enumSet = new EnumSet(Enum32::class);
+        $enumSet->setBinaryBitsetLe("\x0A");
+        $this->assertSame("\x0A\x00\x00\x00", $enumSet->getBinaryBitsetLe());
+    }
+
+    public function testSetBinaryBitsetLeIntOutOfRangeBitsOfExtendedBytes1()
+    {
+        $enumSet = new EnumSet(EnumBasic::class);
+
+        $this->expectException(InvalidArgumentException::class, 'Out-Of-Range');
+        $enumSet->setBinaryBitsetLe("\x0A\xFF\x01");
+    }
+
+    public function testSetBinaryBitsetLeIntOutOfRangeBitsOfExtendedBytes2()
+    {
+        $enumSet = new EnumSet(EnumBasic::class);
+
+        $this->expectException(InvalidArgumentException::class, 'Out-Of-Range');
+        $enumSet->setBinaryBitsetLe("\x01\x01\x01\x01\x01\x01\x01\x01\x01");
+    }
+
+    public function testSetBinaryBitsetLeIntOutOfRangeBitsOfLastValidByte()
+    {
+        // using Enum65 to detect Out-Of-Range bits of last valid byte
+        // Enum65 has max. ordinal number of 2 of the last byte. -> 0001
+        $enumSet   = new EnumSet(Enum31::class);
+        $bitset    = $enumSet->getBinaryBitsetLe();
+        $newBitset = substr($bitset, 0, -1) . "\xFF";
+
+        $this->expectException(InvalidArgumentException::class, 'Out-Of-Range');
+        $enumSet->setBinaryBitsetLe($newBitset);
     }
 
     public function testSetBinaryBitsetBe()
     {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\Enum65');
+        $enumSet = new EnumSet(Enum65::class);
         $enumSet->setBinaryBitsetBe("\x01\x80\x00\x00\x00\x00\x00\x00\x01");
 
         $this->assertTrue($enumSet->contains(Enum65::ONE));
@@ -398,61 +486,51 @@ class EnumSetTest extends TestCase
         $this->assertTrue($enumSet->count() == 3);
     }
 
-    /**
-     * @deprecated
-     */
-    public function testSetBitset()
-    {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\Enum65');
-        $enumSet->setBitset("\x01\x80\x00\x00\x00\x00\x00\x00\x01");
-
-        $this->assertTrue($enumSet->contains(Enum65::ONE));
-        $this->assertFalse($enumSet->contains(Enum65::TWO));
-        $this->assertTrue($enumSet->contains(Enum65::SIXTYFIVE));
-        $this->assertTrue($enumSet->contains(Enum65::SIXTYFOUR));
-        $this->assertTrue($enumSet->count() == 3);
-    }
-
-    public function testSetBinaryBitsetLeShort()
-    {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\Enum65');
-        $enumSet->setBinaryBitsetLe("\x0A");
-        $this->assertSame("\x0A\x00\x00\x00\x00\x00\x00\x00\x00", $enumSet->getBinaryBitsetLe());
-    }
-
-    public function testSetBinaryBitsetLeLong()
-    {
-        $enumSet = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $enumSet->setBinaryBitsetLe("\x0A\xFF\xFF\xFF\xFF\xFF");
-        $this->assertSame("\x0A\xFF", $enumSet->getBinaryBitsetLe());
-    }
-
-    public function testSetBinaryBitsetLeArgumentExceptionIfNotString()
-    {
-        $this->setExpectedException('InvalidArgumentException');
-        
-        $enum = new EnumSet('MabeEnumTest\TestAsset\Enum65');
-        $enum->setBinaryBitsetLe(0);
-    }
-
     public function testSetBinaryBitsetBeArgumentExceptionIfNotString()
     {
-        $this->setExpectedException('InvalidArgumentException');
+        $this->expectException(InvalidArgumentException::class);
         
-        $enum = new EnumSet('MabeEnumTest\TestAsset\Enum65');
+        $enum = new EnumSet(Enum65::class);
         $enum->setBinaryBitsetBe(0);
     }
 
     public function testCountingEmptyEnumEmptySet()
     {
-        $set = new EnumSet('MabeEnumTest\TestAsset\EmptyEnum');
+        $set = new EnumSet(EmptyEnum::class);
         $this->assertSame(0, $set->count());
     }
 
+    public function testCountSingleBit32()
+    {
+        foreach (Enum32::getEnumerators() as $enum) {
+            $enumSet = new EnumSet(Enum32::class);
+            $enumSet->attach($enum);
+            $this->assertSame(1, $enumSet->count());
+        }
+    }
+
+    public function testCountSingleBit64()
+    {
+        foreach (Enum64::getEnumerators() as $enum) {
+            $enumSet = new EnumSet(Enum64::class);
+            $enumSet->attach($enum);
+            $this->assertSame(1, $enumSet->count());
+        }
+    }
+
+    public function testCountSingleBit66()
+    {
+        foreach (Enum66::getEnumerators() as $enum) {
+            $enumSet = new EnumSet(Enum66::class);
+            $enumSet->attach($enum);
+            $this->assertSame(1, $enumSet->count());
+        }
+    }
+
     public function testIsEqual()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $set1 = new EnumSet(EnumBasic::class);
+        $set2 = new EnumSet(EnumBasic::class);
         $this->assertTrue($set1->isEqual($set2));
 
         foreach (EnumBasic::getEnumerators() as $enumerator) {
@@ -466,8 +544,8 @@ class EnumSetTest extends TestCase
 
     public function testIsEqualWrongInstance()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\EnumInheritance');
+        $set1 = new EnumSet(EnumBasic::class);
+        $set2 = new EnumSet(EnumInheritance::class);
         $this->assertFalse($set1->isEqual($set2));
 
         foreach (EnumBasic::getEnumerators() as $enumerator) {
@@ -482,10 +560,10 @@ class EnumSetTest extends TestCase
     /**
      * if $A->isEqual($B) is true then $A->isSubsetOf($B) is also true
      */
-    public function testIsSubsetEqual()
+    public function testIsSubsetIsEqual()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
+        $set1 = new EnumSet(Enum32::class);
+        $set2 = new EnumSet(Enum32::class);
         $this->assertTrue($set1->isSubset($set2));
 
         foreach (Enum32::getEnumerators() as $enumerator) {
@@ -497,8 +575,8 @@ class EnumSetTest extends TestCase
 
     public function testIsSubsetFull()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
+        $set1 = new EnumSet(Enum32::class);
+        $set2 = new EnumSet(Enum32::class);
 
         foreach (Enum32::getEnumerators() as $enumerator) {
             $set2->attach($enumerator);
@@ -508,8 +586,8 @@ class EnumSetTest extends TestCase
 
     public function testIsSubsetFalse()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
+        $set1 = new EnumSet(Enum32::class);
+        $set2 = new EnumSet(Enum32::class);
 
         foreach (Enum32::getEnumerators() as $enumerator) {
             $set1->attach($enumerator);
@@ -519,8 +597,8 @@ class EnumSetTest extends TestCase
 
     public function testIsSubsetWrongInstance()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\EnumInheritance');
+        $set1 = new EnumSet(EnumBasic::class);
+        $set2 = new EnumSet(EnumInheritance::class);
         $this->assertFalse($set1->isSubset($set2));
 
         foreach (EnumBasic::getEnumerators() as $enumerator) {
@@ -535,23 +613,25 @@ class EnumSetTest extends TestCase
     /**
      * if $A->isEqual($B) is true then $A->isSuperset($B) is also true
      */
-    public function testIsSsetEqual()
+    public function testIsSupersetIsEqual()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
+        $set1 = new EnumSet(Enum32::class);
+        $set2 = new EnumSet(Enum32::class);
+        $this->assertTrue($set1->isEqual($set2));
         $this->assertTrue($set1->isSuperset($set2));
 
         foreach (Enum32::getEnumerators() as $enumerator) {
             $set1->attach($enumerator);
             $set2->attach($enumerator);
+            $this->assertTrue($set1->isEqual($set2));
             $this->assertTrue($set1->isSuperset($set2));
         }
     }
 
     public function testIsSupersetFull()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
+        $set1 = new EnumSet(Enum32::class);
+        $set2 = new EnumSet(Enum32::class);
 
         foreach (Enum32::getEnumerators() as $enumerator) {
             $set1->attach($enumerator);
@@ -561,8 +641,8 @@ class EnumSetTest extends TestCase
 
     public function testIsSupersetFalse()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
+        $set1 = new EnumSet(Enum32::class);
+        $set2 = new EnumSet(Enum32::class);
 
         foreach (Enum32::getEnumerators() as $enumerator) {
             $set2->attach($enumerator);
@@ -572,8 +652,8 @@ class EnumSetTest extends TestCase
 
     public function testIsSupersetWrongInstance()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\EnumInheritance');
+        $set1 = new EnumSet(EnumBasic::class);
+        $set2 = new EnumSet(EnumInheritance::class);
         $this->assertFalse($set1->isSuperset($set2));
 
         foreach (EnumBasic::getEnumerators() as $enumerator) {
@@ -585,10 +665,10 @@ class EnumSetTest extends TestCase
         }
     }
 
-    public function testGetOrdinals()
+    public function testGetOrdinalsInt()
     {
-        $set = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $this->assertSame(array(), $set->getOrdinals());
+        $set = new EnumSet(EnumBasic::class);
+        $this->assertSame([], $set->getOrdinals());
 
         foreach (EnumBasic::getConstants() as $value) {
             $set->attach($value);
@@ -597,9 +677,21 @@ class EnumSetTest extends TestCase
         $this->assertSame(range(0, count(EnumBasic::getConstants()) - 1), $set->getOrdinals());
     }
 
-    public function testGetOrdinalsDoesNotEffectIteratorPosition()
+    public function testGetOrdinalsBin()
     {
-        $set = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $set = new EnumSet(Enum66::class);
+        $this->assertSame([], $set->getOrdinals());
+
+        foreach (Enum66::getConstants() as $value) {
+            $set->attach($value);
+        }
+
+        $this->assertSame(range(0, count(Enum66::getConstants()) - 1), $set->getOrdinals());
+    }
+
+    public function testGetOrdinalsIntDoesNotEffectIteratorPosition()
+    {
+        $set = new EnumSet(EnumBasic::class);
         $set->attach(EnumBasic::ONE);
         $set->attach(EnumBasic::TWO);
         $set->next();
@@ -608,10 +700,21 @@ class EnumSetTest extends TestCase
         $this->assertSame(EnumBasic::TWO, $set->current()->getValue());
     }
 
+    public function testGetOrdinalsBinDoesNotEffectIteratorPosition()
+    {
+        $set = new EnumSet(Enum66::class);
+        $set->attach(Enum66::ONE);
+        $set->attach(Enum66::TWO);
+        $set->next();
+
+        $set->getOrdinals();
+        $this->assertSame(Enum66::TWO, $set->current()->getValue());
+    }
+
     public function testGetEnumerators()
     {
-        $set = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $this->assertSame(array(), $set->getEnumerators());
+        $set = new EnumSet(EnumBasic::class);
+        $this->assertSame([], $set->getEnumerators());
 
         foreach (EnumBasic::getConstants() as $value) {
             $set->attach($value);
@@ -622,7 +725,7 @@ class EnumSetTest extends TestCase
 
     public function testGetEnumeratorsDoesNotEffectIteratorPosition()
     {
-        $set = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $set = new EnumSet(EnumBasic::class);
         $set->attach(EnumBasic::ONE);
         $set->attach(EnumBasic::TWO);
         $set->next();
@@ -633,8 +736,8 @@ class EnumSetTest extends TestCase
 
     public function testGetValues()
     {
-        $set = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $this->assertSame(array(), $set->getValues());
+        $set = new EnumSet(EnumBasic::class);
+        $this->assertSame([], $set->getValues());
 
         foreach (EnumBasic::getConstants() as $value) {
             $set->attach($value);
@@ -645,7 +748,7 @@ class EnumSetTest extends TestCase
 
     public function testGetValuesDoesNotEffectIteratorPosition()
     {
-        $set = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $set = new EnumSet(EnumBasic::class);
         $set->attach(EnumBasic::ONE);
         $set->attach(EnumBasic::TWO);
         $set->next();
@@ -656,8 +759,8 @@ class EnumSetTest extends TestCase
 
     public function testGetNames()
     {
-        $set = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $this->assertSame(array(), $set->getNames());
+        $set = new EnumSet(EnumBasic::class);
+        $this->assertSame([], $set->getNames());
 
         foreach (EnumBasic::getConstants() as $value) {
             $set->attach($value);
@@ -668,7 +771,7 @@ class EnumSetTest extends TestCase
 
     public function testGetNamesDoesNotEffectIteratorPosition()
     {
-        $set = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $set = new EnumSet(EnumBasic::class);
         $set->attach(EnumBasic::ONE);
         $set->attach(EnumBasic::TWO);
         $set->next();
@@ -679,131 +782,108 @@ class EnumSetTest extends TestCase
 
     public function testUnion()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $set1 = new EnumSet(EnumBasic::class);
         $set1->attach(EnumBasic::ONE);
         $set1->attach(EnumBasic::TWO);
 
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $set2 = new EnumSet(EnumBasic::class);
         $set2->attach(EnumBasic::TWO);
         $set2->attach(EnumBasic::THREE);
+        $set2->attach(EnumBasic::FOUR);
 
-        $set3 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $set3->attach(EnumBasic::THREE);
-        $set3->attach(EnumBasic::FOUR);
-
-        $rs = $set1->union($set2, $set3);
-        $this->assertSame(array(
+        $rs = $set1->union($set2);
+        $this->assertSame([
             EnumBasic::ONE,
             EnumBasic::TWO,
             EnumBasic::THREE,
             EnumBasic::FOUR,
-        ), $rs->getValues());
+        ], $rs->getValues());
     }
 
     public function testUnionThrowsInvalidArgumentException()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
+        $set1 = new EnumSet(EnumBasic::class);
+        $set2 = new EnumSet(Enum32::class);
 
-        $this->setExpectedException('InvalidArgumentException');
+        $this->expectException(InvalidArgumentException::class);
         $set1->union($set2);
     }
 
     public function testIntersect()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $set1 = new EnumSet(EnumBasic::class);
         $set1->attach(EnumBasic::ONE);
         $set1->attach(EnumBasic::TWO);
         $set1->attach(EnumBasic::THREE);
 
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $set2 = new EnumSet(EnumBasic::class);
         $set2->attach(EnumBasic::TWO);
         $set2->attach(EnumBasic::THREE);
         $set2->attach(EnumBasic::FOUR);
 
-        $set3 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $set3->attach(EnumBasic::THREE);
-        $set3->attach(EnumBasic::FOUR);
-        $set3->attach(EnumBasic::FIVE);
-
-        $rs = $set1->intersect($set2, $set3);
-        $this->assertSame(array(
-            EnumBasic::THREE,
-        ), $rs->getValues());
+        $rs = $set1->intersect($set2);
+        $this->assertSame([EnumBasic::TWO, EnumBasic::THREE], $rs->getValues());
     }
 
     public function testIntersectThrowsInvalidArgumentException()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
+        $set1 = new EnumSet(EnumBasic::class);
+        $set2 = new EnumSet(Enum32::class);
 
-        $this->setExpectedException('InvalidArgumentException');
+        $this->expectException(InvalidArgumentException::class);
         $set1->intersect($set2);
     }
 
     public function testDiff()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $set1 = new EnumSet(EnumBasic::class);
         $set1->attach(EnumBasic::ONE);
         $set1->attach(EnumBasic::TWO);
         $set1->attach(EnumBasic::THREE);
 
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $set2 = new EnumSet(EnumBasic::class);
         $set2->attach(EnumBasic::TWO);
         $set2->attach(EnumBasic::THREE);
         $set2->attach(EnumBasic::FOUR);
 
-        $set3 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $set3->attach(EnumBasic::THREE);
-        $set3->attach(EnumBasic::FOUR);
-        $set3->attach(EnumBasic::FIVE);
-
-        $rs = $set1->diff($set2, $set3);
-        $this->assertSame(array(
-            EnumBasic::ONE,
-        ), $rs->getValues());
+        $rs = $set1->diff($set2);
+        $this->assertSame([EnumBasic::ONE], $rs->getValues());
     }
 
     public function testDiffThrowsInvalidArgumentException()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
+        $set1 = new EnumSet(EnumBasic::class);
+        $set2 = new EnumSet(Enum32::class);
 
-        $this->setExpectedException('InvalidArgumentException');
+        $this->expectException(InvalidArgumentException::class);
         $set1->diff($set2);
     }
 
     public function testSymDiff()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $set1 = new EnumSet(EnumBasic::class);
         $set1->attach(EnumBasic::ONE);
         $set1->attach(EnumBasic::TWO);
         $set1->attach(EnumBasic::THREE);
 
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
+        $set2 = new EnumSet(EnumBasic::class);
         $set2->attach(EnumBasic::TWO);
         $set2->attach(EnumBasic::THREE);
         $set2->attach(EnumBasic::FOUR);
 
-        $set3 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $set3->attach(EnumBasic::THREE);
-        $set3->attach(EnumBasic::FOUR);
-        $set3->attach(EnumBasic::FIVE);
-
-        $rs = $set1->symDiff($set2, $set3);
-        $this->assertSame(array(
+        $rs = $set1->symDiff($set2);
+        $this->assertSame([
             EnumBasic::ONE,
             EnumBasic::FOUR,
-            EnumBasic::FIVE,
-        ), $rs->getValues());
+        ], $rs->getValues());
     }
 
     public function testSymDiffThrowsInvalidArgumentException()
     {
-        $set1 = new EnumSet('MabeEnumTest\TestAsset\EnumBasic');
-        $set2 = new EnumSet('MabeEnumTest\TestAsset\Enum32');
+        $set1 = new EnumSet(EnumBasic::class);
+        $set2 = new EnumSet(Enum32::class);
 
-        $this->setExpectedException('InvalidArgumentException');
+        $this->expectException(InvalidArgumentException::class);
         $set1->symDiff($set2);
     }
 }
